package com.excellapps.ideaactions;

import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.actionSystem.DataConstants;
import com.intellij.openapi.actionSystem.DataContext;
import com.intellij.openapi.editor.CaretModel;
import com.intellij.openapi.editor.Editor;
import com.intellij.openapi.editor.actionSystem.EditorAction;
import com.intellij.openapi.editor.actionSystem.EditorWriteActionHandler;
import com.intellij.openapi.project.Project;

/**
 * Pastes the current snippet into current editor (if opened)
 * 
 * @author Anki R Nelaturu
 */
public class PasteAction extends EditorAction {
    public void update(AnActionEvent e) {
        try {
            e.getPresentation().setEnabled(true);
        } catch (Exception ex) {
            //
        }
    }

    public PasteAction() {
        super(new EditorWriteActionHandler(){
            public void executeWriteAction(Editor editor, DataContext dataContext) {
                final Project project = (Project) dataContext.getData(DataConstants.PROJECT);
                if (project == null) {
                    return;
                }

                final ISnippetPlugin consolePlugin = (ISnippetPlugin) project.getComponent(ISnippetPlugin.class);
                if (consolePlugin == null) {
                    return;
                }
                SearchPanel searchPanel = consolePlugin.getSearchPanel();
                String currentSnippet = searchPanel.getCurrentSnippet();

                if (editor != null) {
                    CaretModel caretModel = editor.getCaretModel();
                    com.intellij.openapi.editor.Document document = editor.getDocument();
                    if (caretModel != null && document != null ) {
                        int offset = caretModel.getOffset();
                        document.insertString(offset, currentSnippet);
                    }
                }
            }
        });
    }
}
