package com.excellapps.ideaactions;

import com.excellapps.isnippet.ISnippet;
import com.excellapps.isnippet.ISnippetSearchAgent;
import com.excellapps.isnippet.ISnippetSearchAgentsFactory;
import com.excellapps.isnippet.tree.RootNode;
import com.excellapps.isnippet.tree.ResultNode;
import com.excellapps.isnippet.tree.SearchAgentNode;
import com.intellij.openapi.actionSystem.ActionManager;
import com.intellij.openapi.actionSystem.ActionToolbar;
import com.intellij.openapi.actionSystem.DefaultActionGroup;
import com.intellij.uiDesigner.core.GridLayoutManager;
import com.intellij.uiDesigner.core.GridConstraints;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.TreeSelectionListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.text.BadLocationException;
import javax.swing.tree.DefaultTreeModel;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.StringTokenizer;
import java.net.URL;
import java.io.IOException;

/**
 * The UI.
 *
 * @author Anki R Nelaturu
 */
public class SearchPanel {
    public JPanel mainPanel;
    private JTextField searchForTextField;
    private JButton goButton;
    private JTabbedPane tabbedPane1;
    private JCheckBox detailedSearchCheckBox;
    private JCheckBox inCodeOnlyCheckBox;
    private JEditorPane htmlPane;
    private JEditorPane codeEditorPane;
    private JPanel codePanel;
    private JTree resultsTree;
    private JProgressBar progressBar;
    private RootNode rootNode = new RootNode();
    private ArrayList<String> currentCodeSnippets = new ArrayList<String>();
    private int currentCodeSnippetIndex = 0;
    private JPanel panel1;

    public SearchPanel(ISnippetPlugin thePlugin) {
        resultsTree.setModel(new DefaultTreeModel(rootNode));

        final ActionManager actionManager = ActionManager.getInstance();

        final DefaultActionGroup actionGroup = (DefaultActionGroup) actionManager.getAction("iSnippet.ToolGroup");
        final ActionToolbar actionToolbar = actionManager.createActionToolbar("iSnippet.ToolGroup", actionGroup, false);
        codePanel.add(actionToolbar.getComponent(), BorderLayout.WEST);

        detailedSearchCheckBox.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent changeEvent) {
                if (detailedSearchCheckBox.isSelected()) {
                    inCodeOnlyCheckBox.setEnabled(true);
                } else {
                    inCodeOnlyCheckBox.setEnabled(false);
                    inCodeOnlyCheckBox.setSelected(false);
                }
            }
        });
        searchForTextField.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent actionEvent) {
                search();
            }
        });
        goButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent actionEvent) {
                search();
            }
        });
//        resultsList.addListSelectionListener(new ListSelectionListener() {
//            public void valueChanged(ListSelectionEvent listSelectionEvent) {
//                ISnippet snippet = (ISnippet) resultsList.getSelectedValue();
//                htmlPane.setText(snippet.getHTML());
//                htmlPane.setCaretPosition(0);
//                currentCodeSnippets = snippet.getCodeSnippets();
//                currentCodeSnippetIndex = 0;
//                refreshCodeTextArea();
//            }
//        });
        resultsTree.addTreeSelectionListener(new TreeSelectionListener() {
            public void valueChanged(final TreeSelectionEvent treeSelectionEvent) {
                new Thread(new Runnable() {
                    public void run() {
                        startProgress();
                        Object pe = treeSelectionEvent.getPath().getLastPathComponent();
                        if (pe instanceof ResultNode) {
                            displayResultInfo((ResultNode) pe);
                        } else if (pe instanceof SearchAgentNode) {
                            displayAgentInfo((SearchAgentNode) pe);
                        }
                        endProgress();
                    }
                }).start();
            }
        });
    }

    private void startProgress() {
        searchForTextField.setEnabled(false);
        goButton.setEnabled(false);
        resultsTree.setEnabled(false);
        progressBar.setIndeterminate(true);
    }

    private void endProgress() {
        searchForTextField.setEnabled(true);
        goButton.setEnabled(true);
        resultsTree.setEnabled(true);
        progressBar.setIndeterminate(false);
    }

    private void displayAgentInfo(SearchAgentNode searchAgentNode) {
        ISnippetSearchAgent agent = searchAgentNode.getAgent();
        codeEditorPane.setText(agent.getDescription());

        try {
            htmlPane.setPage(new URL(agent.getHomePageURL()));
        } catch (IOException e) {
            htmlPane.setText(e.getMessage());
            e.printStackTrace();
        }
    }

    private void displayResultInfo(ResultNode resultNode) {
        ISnippet snippet = resultNode.getSnippet();
        htmlPane.setText(snippet.getHTML());
        htmlPane.setCaretPosition(0);
        currentCodeSnippets = snippet.getCodeSnippets();
        currentCodeSnippetIndex = 0;
        refreshCodeTextArea();
    }

    public String getCurrentSnippet() {
        try {
            return codeEditorPane.getDocument().getText(0, codeEditorPane.getDocument().getLength());
        } catch (BadLocationException e) {
            e.printStackTrace();
        }
        return "";
    }

    public void nextCodeSnippet() {
        if (currentCodeSnippetIndex < currentCodeSnippets.size() - 1) {
            currentCodeSnippetIndex++;
            refreshCodeTextArea();
        }
    }

    public void previousCodeSnippet() {
        if (currentCodeSnippetIndex > 0) {
            currentCodeSnippetIndex--;
            refreshCodeTextArea();
        }
    }

    public boolean hasNext() {
        return currentCodeSnippetIndex < currentCodeSnippets.size() - 1;
    }

    public boolean hasPrevious() {
        return currentCodeSnippetIndex > 0;
    }

    private void refreshCodeTextArea() {
        if (currentCodeSnippets.size() <= 0) {
            codeEditorPane.setText("<html><body><h1>No code snippets found. Sorry</ht></body></html>");
            return;
        }

        codeEditorPane.setText(currentCodeSnippets.get(currentCodeSnippetIndex));
        codeEditorPane.setCaretPosition(0);
    }

    private void search() {
        new Thread(new Runnable() {
            public void run() {
                startProgress();
                search_internal();
                endProgress();
            }
        }).start();
    }

    private void search_internal() {
        String str = searchForTextField.getText().trim();
        StringTokenizer st = new StringTokenizer(str, " ");
        int count = st.countTokens();
        String[] keywords = new String[count];
        for (int i = 0; i < count; i++) {
            keywords[i] = st.nextToken();
        }

        rootNode.removeAllChildren();
        ArrayList<ISnippet> matches = new ArrayList<ISnippet>();
        ArrayList<ISnippetSearchAgent> agents = ISnippetSearchAgentsFactory.getSnippetManagers();
        SearchAgentNode node = null;
        for (ISnippetSearchAgent agent : agents) {
            node = new SearchAgentNode(agent, matches.size());
            matches = agent.matchingList(keywords, detailedSearchCheckBox.isSelected(), inCodeOnlyCheckBox.isSelected());
            for (ISnippet snippet : matches) {
                node.add(new ResultNode(snippet));
                System.out.println(snippet.getSummary());
            }
            rootNode.add(node);
        }

        resultsTree.invalidate();
        resultsTree.updateUI();
        resultsTree.validate();
        int rc = resultsTree.getRowCount();
        for (int i = 0; i < rc; i++) {
            resultsTree.expandRow(i);
        }
    }

    {
// GUI initializer generated by IntelliJ IDEA GUI Designer
// >>> IMPORTANT!! <<<
// DO NOT EDIT OR ADD ANY CODE HERE!
        $$$setupUI$$$();
    }

    /**
     * Method generated by IntelliJ IDEA GUI Designer
     * >>> IMPORTANT!! <<<
     * DO NOT edit this method OR call it in your code!
     *
     * @noinspection HardCodedStringLiteral
     */
    private void $$$setupUI$$$() {
        mainPanel = new JPanel();
        mainPanel.setLayout(new GridLayoutManager(1, 1, new Insets(0, 0, 0, 0), -1, -1));
        final JSplitPane splitPane1 = new JSplitPane();
        splitPane1.setDividerLocation(238);
        mainPanel.add(splitPane1, new GridConstraints(0, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_BOTH, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, null, new Dimension(200, 200), null, 0, false));
        final JPanel panel2 = new JPanel();
        panel2.setLayout(new GridLayoutManager(3, 1, new Insets(0, 0, 0, 0), -1, -1));
        splitPane1.setLeftComponent(panel2);
        final JPanel panel3 = new JPanel();
        panel3.setLayout(new GridLayoutManager(2, 1, new Insets(0, 0, 0, 0), -1, -1));
        panel2.add(panel3, new GridConstraints(0, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_BOTH, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, null, null, null, 0, false));
        panel3.setBorder(BorderFactory.createTitledBorder(BorderFactory.createLineBorder(new Color(-16777216)), "Search"));
        final JPanel panel4 = new JPanel();
        panel4.setLayout(new GridLayoutManager(1, 3, new Insets(0, 0, 0, 0), -1, -1));
        panel3.add(panel4, new GridConstraints(0, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_BOTH, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, null, null, null, 0, false));
        final JLabel label1 = new JLabel();
        label1.setText("Find:");
        panel4.add(label1, new GridConstraints(0, 0, 1, 1, GridConstraints.ANCHOR_WEST, GridConstraints.FILL_NONE, GridConstraints.SIZEPOLICY_FIXED, GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
        goButton = new JButton();
        goButton.setText("Go");
        panel4.add(goButton, new GridConstraints(0, 2, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_HORIZONTAL, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
        searchForTextField = new JTextField();
        panel4.add(searchForTextField, new GridConstraints(0, 1, 1, 1, GridConstraints.ANCHOR_WEST, GridConstraints.FILL_BOTH, 1, GridConstraints.SIZEPOLICY_FIXED, null, new Dimension(150, -1), null, 0, false));
        final JPanel panel5 = new JPanel();
        panel5.setLayout(new GridLayoutManager(1, 2, new Insets(0, 0, 0, 0), -1, -1));
        panel3.add(panel5, new GridConstraints(1, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_BOTH, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, null, null, null, 0, false));
        detailedSearchCheckBox = new JCheckBox();
        detailedSearchCheckBox.setText("Detailed Search");
        panel5.add(detailedSearchCheckBox, new GridConstraints(0, 0, 1, 1, GridConstraints.ANCHOR_WEST, GridConstraints.FILL_NONE, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
        inCodeOnlyCheckBox = new JCheckBox();
        inCodeOnlyCheckBox.setEnabled(false);
        inCodeOnlyCheckBox.setText("In Code Only");
        panel5.add(inCodeOnlyCheckBox, new GridConstraints(0, 1, 1, 1, GridConstraints.ANCHOR_WEST, GridConstraints.FILL_NONE, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
        final JScrollPane scrollPane1 = new JScrollPane();
        panel2.add(scrollPane1, new GridConstraints(1, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_BOTH, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_WANT_GROW, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_WANT_GROW, null, null, null, 0, false));
        scrollPane1.setBorder(BorderFactory.createTitledBorder(BorderFactory.createLineBorder(new Color(-16777216)), "Results"));
        resultsTree = new JTree();
        resultsTree.setRootVisible(false);
        scrollPane1.setViewportView(resultsTree);
        progressBar = new JProgressBar();
        progressBar.setIndeterminate(false);
        panel2.add(progressBar, new GridConstraints(2, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_HORIZONTAL, GridConstraints.SIZEPOLICY_WANT_GROW, GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
        final JPanel panel6 = new JPanel();
        panel6.setLayout(new GridLayoutManager(1, 1, new Insets(0, 0, 0, 0), -1, -1));
        splitPane1.setRightComponent(panel6);
        tabbedPane1 = new JTabbedPane();
        panel6.add(tabbedPane1, new GridConstraints(0, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_BOTH, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, null, new Dimension(200, 200), null, 0, false));
        codePanel = new JPanel();
        codePanel.setLayout(new BorderLayout(0, 0));
        tabbedPane1.addTab("Code Snippets", codePanel);
        final JScrollPane scrollPane2 = new JScrollPane();
        codePanel.add(scrollPane2, BorderLayout.CENTER);
        codeEditorPane = new JEditorPane();
        codeEditorPane.setContentType("text/html");
        codeEditorPane.setEditable(false);
        scrollPane2.setViewportView(codeEditorPane);
        final JScrollPane scrollPane3 = new JScrollPane();
        tabbedPane1.addTab("Detailed Description", scrollPane3);
        htmlPane = new JEditorPane();
        htmlPane.setContentType("text/html");
        htmlPane.setEditable(false);
        scrollPane3.setViewportView(htmlPane);
    }

    public JComponent $$$getRootComponent$$$() {
        return mainPanel;
    }
}
