package com.excellapps.isnippet.impl.jda;

import com.excellapps.isnippet.ISnippet;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.ArrayList;

/**
 * This is the Snippet implementation to parse and hold the actual information.
 *
 * @author Anki R Nelaturu
 */
public class JDASnippet implements ISnippet {
	public static final String MAIN_URL = "http://javaalmanac.com/egs/";
	private String href;
	private String description;
	private String url = null;
	private String html;
	private ArrayList<String> codeSnippets = new ArrayList<String>();
	private String descriptionLower;

	public JDASnippet(String href, String description) {
		this.href = href.toLowerCase();
		this.description = description;
		this.descriptionLower = description.toLowerCase();
		url = MAIN_URL + href;
	}

	public boolean matches(String[] keywords, boolean detailedSearch, boolean inCodeOnly) {
		for (String keyword : keywords) {
			if (href.indexOf(keyword.toLowerCase()) != -1
					|| descriptionLower.indexOf(keyword.toLowerCase()) != -1) {
                if(detailedSearch) {
                    return getHTML().indexOf(keyword.toLowerCase()) != -1;
                }
				return true;
			}
		}

		return false;
	}

	public String getURL() {
		return url;
	}

	private void prepareData() {
		if (html != null) {
			return;
		}
		try {
			html = getContent();

			int index = html.indexOf(description + "</h3>");
			html = "<html><body><h3>" + html.substring(index);

			index = html
					.indexOf("<P><table BORDER=\"0\" CELLPADDING=\"2\" CELLSPACING=\"0\" width=\"600\">");
			html = html.substring(0, index);
			html += "</body></html>";

            parseCodeSnippets();
        } catch (Exception e) {
			e.printStackTrace();
		}
	}

    private void parseCodeSnippets() {
        int index = html.indexOf("<pre>");
        int end = 0;
        if(index == -1) {
            return;
        }

        while(index != -1) {
            end = html.indexOf("</pre>", index);
            codeSnippets.add("<html><body>" + html.substring(index, end + 5) + "</body></html");
            index = html.indexOf("<pre>", end);
        }
    }

    public String getSummary() {
        return description;
    }

    public String toString() {
        return description;
    }

    public String getHTML() {
		prepareData();
		return html;
	}

	public ArrayList<String> getCodeSnippets() {
        prepareData();
        return codeSnippets;
	}

	private String getContent() throws Exception {
        System.out.println("URL: " + url);
        BufferedReader in = new BufferedReader(new InputStreamReader(new URL(
				url).openStream()));

		String line = null;
		StringBuilder sb = new StringBuilder();

		while ((line = in.readLine()) != null)
			sb.append(line).append('\n');

		in.close();

		return sb.toString();
	}
}
