package com.excellapps.isnippet.impl.jda;

import com.excellapps.isnippet.ISnippet;
import com.excellapps.isnippet.ISnippetSearchAgent;

import java.util.ArrayList;
import java.util.Properties;

/**
 * Sample implementation of SnippetSearchAgent. THis agent searches the Java Developers almanac site.
 * To make the implementation simple, this agent uses predefined URLs (from snippet.properties) to search for the given
 * keywords.
 *
 * @author Anki R Nelaturu
 */
public class JDASnippetSearchAgent implements ISnippetSearchAgent {
	private ArrayList<JDASnippet> urls = new ArrayList<JDASnippet>();

	public JDASnippetSearchAgent() throws Exception {
		Properties p = new Properties();
		p.load(JDASnippetSearchAgent.class.getResourceAsStream("/com/excellapps/isnippet/impl/jda/snippets.properties"));

		int count = Integer.parseInt(p.getProperty("snippets.count"));

		for(int i = 0; i < count; i++) {
			urls.add(new JDASnippet(p.getProperty("snippet.href." + i), p.getProperty("snippet.description." + i)));
		}
	}

	public ArrayList<ISnippet> matchingList(String[] keywords, boolean detailedSearch, boolean inCodeOnly) {
		ArrayList<ISnippet> matchingList = new ArrayList<ISnippet>();

		for(JDASnippet url : urls) {
			if(url.matches(keywords, detailedSearch, inCodeOnly)) {
				matchingList.add(url);
			}
		}

		return matchingList;
	}

	public String getDescription() {
		return "This website supplements the book The Java Developers Almanac 1.4. All the code examples from the book are made available here for you to copy and paste into your programs.";
	}

	public String getHomePageURL() {
		return "http://javaalmanac.com/";
	}

	public String getDisplayName() {
		return "The Java Developers Almanac 1.4";
	}

}
